package vip.devkit.debugkit;

import android.content.Context;

import java.util.ArrayList;
import java.util.List;

import vip.devkit.debugkit.kit.GroupKitItem;
import vip.devkit.debugkit.kit.IKit;
import vip.devkit.debugkit.kit.KitItem;
import vip.devkit.debugkit.kit.debug.AppInfo;
import vip.devkit.debugkit.kit.debug.BaseUrlManger;
import vip.devkit.debugkit.kit.debug.CachesManager;
import vip.devkit.debugkit.kit.debug.CrashManger;
import vip.devkit.debugkit.kit.debug.DataExport;
import vip.devkit.debugkit.kit.debug.FileExplorer;
import vip.devkit.debugkit.kit.debug.WebBrowse;
import vip.devkit.debugkit.kit.debug.LogManager;
import vip.devkit.debugkit.kit.debug.PermissionManager;
import vip.devkit.debugkit.kit.debug.Simulation;
import vip.devkit.debugkit.kit.performance.Battery;
import vip.devkit.debugkit.kit.performance.CPU;
import vip.devkit.debugkit.kit.performance.FTPS;
import vip.devkit.debugkit.kit.performance.RAM;
import vip.devkit.debugkit.kit.performance.Time;
import vip.devkit.debugkit.kit.performance.Traffic;
import vip.devkit.debugkit.kit.system.CpuKit;
import vip.devkit.debugkit.kit.system.HardwareKit;
import vip.devkit.debugkit.kit.system.NetworkKit;
import vip.devkit.debugkit.kit.system.ScreenKit;
import vip.devkit.debugkit.kit.system.SysInfo;
import vip.devkit.debugkit.kit.ue.ColorPicker;
import vip.devkit.debugkit.kit.ue.UE;
import vip.devkit.debugkit.util.ResUtil;

/**
 * Package    Name: vip.devkit.debugkit
 * File       Name: DataSource.java
 * Create     Date: 2019/7/1 17:24
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class DataSource {

    private List<GroupKitItem> mItemList = new ArrayList<>();
    private List<KitItem> mDiyKitItems = new ArrayList<>();
    private Context mContext;

    public List<GroupKitItem> getKitList() {
        return mItemList;
    }

    public DataSource(Context context) {
        mContext = context;
        initKit();
    }

    private void initKit() {
        sysKit();
        debugKit();
        developerKit();
        performanceKit();
        ueKit();
        diyKit();
    }


    public void sysKit() {
        GroupKitItem sys = new GroupKitItem();
        sys.mGroupName = ResUtil.getString(mContext, R.string.dk_sys_system);

        List<IKit> d = new ArrayList<>();
        d.add(new SysInfo());
        d.add(new ScreenKit());
        d.add(new NetworkKit());
        d.add(new HardwareKit());
        d.add(new CpuKit());

        List<KitItem> kitItems = new ArrayList<>();
        for (IKit kit : d) {
            kitItems.add(new KitItem(kit));
        }
        sys.mKitList = kitItems;
        mItemList.add(sys);

    }


    public void debugKit() {
        GroupKitItem mGroupBean = new GroupKitItem();
        mGroupBean.mGroupName = ResUtil.getString(mContext, R.string.dk_debug);


        List<IKit> d = new ArrayList<>();
        d.add(new AppInfo());
        d.add(new FileExplorer());
        d.add(new LogManager());
        d.add(new PermissionManager());
        d.add(new BaseUrlManger());
        d.add(new CachesManager());
        d.add(new CrashManger());
        d.add(new Simulation());
        d.add(new DataExport());
        d.add(new WebBrowse());


        List<KitItem> kitItems = new ArrayList<>();
        for (IKit kit : d) {
            kitItems.add(new KitItem(kit));
        }
        mGroupBean.mKitList = kitItems;
        mItemList.add(mGroupBean);



    }

    public void developerKit() {
        GroupKitItem mGroupBean = new GroupKitItem();
        mGroupBean.mGroupName = ResUtil.getString(mContext, R.string.dk_developer);

        List<IKit> d = new ArrayList<>();
        d.add(new SysInfo());
        d.add(new ScreenKit());
        d.add(new NetworkKit());
        d.add(new HardwareKit());
        d.add(new CpuKit());

        List<KitItem> kitItems = new ArrayList<>();
        for (IKit kit : d) {
            kitItems.add(new KitItem(kit));
        }
        mGroupBean.mKitList = kitItems;
        mItemList.add(mGroupBean);

    }

    public void performanceKit() {
        GroupKitItem mGroupBean = new GroupKitItem();
        mGroupBean.mGroupName = ResUtil.getString(mContext, R.string.dk_performance);

        List<IKit> d = new ArrayList<>();
        d.add(new FTPS());
        d.add(new CPU());
        d.add(new RAM());
        d.add(new Traffic());
        d.add(new Battery());
        d.add(new Time());

        List<KitItem> kitItems = new ArrayList<>();
        for (IKit kit : d) {
            kitItems.add(new KitItem(kit));
        }
        mGroupBean.mKitList = kitItems;
        mItemList.add(mGroupBean);

    }

    public void ueKit() {
        GroupKitItem mGroupBean = new GroupKitItem();
        mGroupBean.mGroupName = ResUtil.getString(mContext, R.string.dk_ue);

        List<IKit> d = new ArrayList<>();
        d.add(new ColorPicker());
        d.add(new UE());

        List<KitItem> kitItems = new ArrayList<>();
        for (IKit kit : d) {
            kitItems.add(new KitItem(kit));
        }
        mGroupBean.mKitList = kitItems;
        mItemList.add(mGroupBean);

    }


    public void diyKit() {
        if (mDiyKitItems.size() > 0) {
            GroupKitItem mGroupBean = new GroupKitItem();
            mGroupBean.mGroupName = ResUtil.getString(mContext, R.string.dk_diy);
            mGroupBean.mKitList = mDiyKitItems;
        }
    }


    public void add(IKit mIKit) {
        mDiyKitItems.add(new KitItem(mIKit));
    }

    public void add(List<IKit> list) {
        for (IKit kit : list) {
            mDiyKitItems.add(new KitItem(kit));
        }
    }
}
