package vip.devkit.debugkit.bean;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.drawable.Drawable;
import android.os.Build;

import androidx.annotation.Keep;

import vip.devkit.debugkit.util.AppUtil;
import vip.devkit.debugkit.util.DeviceUtils;
import vip.devkit.debugkit.util.FileUtil;

/**
 * Package    Name: vip.devkit.debugkit.bean
 * File       Name: AppInfoBean.java
 * Create     Date: 2019/7/8 13:56
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class AppInfoBean {

    @Keep // App 包名
    private String appPackName;
    @Keep // App 应用名
    private String appName;
    @Keep // App 图标
    private transient Drawable appIcon;
    @Keep // App 类型
    private AppType appType;
    @Keep // App 版本号
    private int versionCode;
    @Keep // App 版本名
    private String versionName;
    @Keep // App 首次安装时间
    private long firstInstallTime;
    @Keep // App 最后一次更新时间
    private long lastUpdateTime;
    @Keep // App 地址
    private String sourceDir;
    @Keep // APK 大小
    private long apkSize;
    @Keep //Sign MD5
    private String apkSign;
    @Keep //最小支持
    private int minSdkVersion = -1;
    @Keep //兼容
    private int targetSdkVersion = -1;

    @Keep //activity 数量
    private int activityCount;
    @Keep //service 数量
    private int serviceCount;
    @Keep //receiver 数量
    private int receiverCount;
    @Keep //provider 数量
    private int providerCount;
    @Keep //permission 数量
    private int permissionCount;


    private Context context;


    /**
     * 初始化 AppInfoBean
     */
    protected AppInfoBean() {

        init();
    }

    public AppInfoBean(Context context) {
        this.context = context;
        init();
    }

    private void init() {
        //GET_CONFIGURATIONS
        PackageInfo packageInfo = DeviceUtils.getPackageInfo(context);

        PackageManager packageManager = context.getPackageManager();
        // App 包名
        appPackName = packageInfo.applicationInfo.packageName;
        // App 应用名
        appName = packageManager.getApplicationLabel(packageInfo.applicationInfo).toString();
        // App 图标
        appIcon = packageManager.getApplicationIcon(packageInfo.applicationInfo);
        // App 类型
        appType = AppInfoBean.getAppType(packageInfo);
        // App 版本号
        versionCode = packageInfo.versionCode;
        // App 版本名
        versionName = packageInfo.versionName;
        // App 首次安装时间
        firstInstallTime = packageInfo.firstInstallTime;
        // App 最后一次更新时间
        lastUpdateTime = packageInfo.lastUpdateTime;
        // App 地址
        sourceDir = packageInfo.applicationInfo.sourceDir;
        // APK 大小
//        apkSize = FileUtil.getFileSize(context,sourceDir);
        // APK sign
        apkSign = AppUtil.getSign(context, packageInfo.packageName);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            minSdkVersion = context.getApplicationInfo().minSdkVersion;
        }
        targetSdkVersion = context.getApplicationInfo().targetSdkVersion;
        PackageInfo pia = DeviceUtils.getPackageInfo(context, PackageManager.GET_ACTIVITIES);
        if (pia.activities != null) {
            activityCount = pia.activities.length;
        }

        PackageInfo pis = DeviceUtils.getPackageInfo(context, PackageManager.GET_SERVICES);
        if (pis.services != null) {
            serviceCount = pis.services.length;
        }

        PackageInfo pir = DeviceUtils.getPackageInfo(context, PackageManager.GET_RECEIVERS);
        if (pir.receivers != null) {
            receiverCount = pir.receivers.length;
        }
        PackageInfo pip = DeviceUtils.getPackageInfo(context, PackageManager.GET_PROVIDERS);
        if (pip.providers != null) {
            providerCount = pip.providers.length;
        }

        PackageInfo permissions = DeviceUtils.getPackageInfo(context, PackageManager.GET_PERMISSIONS);
        if (permissions.requestedPermissions != null) {
            permissionCount = permissions.requestedPermissions.length;
        }
    }


    /**
     * detail: 应用类型
     */
    public enum AppType {

        USER, // 用户 App

        SYSTEM, // 系统 App

        ALL // 全部 App
    }

    public String getAppPackName() {
        return appPackName;
    }

    public String getAppName() {
        return appName;
    }

    public Drawable getAppIcon() {
        return appIcon;
    }

    public AppType getAppType() {
        return appType;
    }

    public int getVersionCode() {
        return versionCode;
    }

    public String getVersionName() {
        return versionName;
    }

    public long getFirstInstallTime() {
        return firstInstallTime;
    }

    public long getLastUpdateTime() {
        return lastUpdateTime;
    }

    public String getSourceDir() {
        return sourceDir;
    }

    public long getApkSize() {
        return apkSize;
    }

    public String getApkSign() {
        return apkSign;
    }

    public int getMinSdkVersion() {
        return minSdkVersion;
    }

    public int getTargetSdkVersion() {
        return targetSdkVersion;
    }

    public int getActivityCount() {
        return activityCount;
    }

    public int getServiceCount() {
        return serviceCount;
    }

    public int getReceiverCount() {
        return receiverCount;
    }

    public int getProviderCount() {
        return providerCount;
    }

    public int getPermissionCount() {
        return permissionCount;
    }

    /**
     * 获取 App 类型
     *
     * @param packageInfo {@link PackageInfo}
     * @return {@link AppType} 应用类型
     */
    public static AppType getAppType(final PackageInfo packageInfo) {
        if (!isSystemApp(packageInfo) && !isSystemUpdateApp(packageInfo)) {
            return AppType.USER;
        }
        return AppType.SYSTEM;
    }

    /**
     * 是否系统程序
     *
     * @param packageInfo {@link PackageInfo}
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isSystemApp(final PackageInfo packageInfo) {
        return ((packageInfo.applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) != 0);
    }

    /**
     * 是否系统程序被手动更新后, 也成为第三方应用程序
     *
     * @param packageInfo {@link PackageInfo}
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isSystemUpdateApp(final PackageInfo packageInfo) {
        return ((packageInfo.applicationInfo.flags & ApplicationInfo.FLAG_UPDATED_SYSTEM_APP) != 0);
    }

    @Override
    public String toString() {
        return "AppInfoBean{" +
                "appPackName='" + appPackName + '\'' +
                ", appName='" + appName + '\'' +
                ", appIcon=" + appIcon +
                ", appType=" + appType +
                ", versionCode=" + versionCode +
                ", versionName='" + versionName + '\'' +
                ", firstInstallTime=" + firstInstallTime +
                ", lastUpdateTime=" + lastUpdateTime +
                ", sourceDir='" + sourceDir + '\'' +
                ", apkSize=" + apkSize +
                ", apkSign='" + apkSign + '\'' +
                ", minSdkVersion='" + targetSdkVersion + '\'' +
                ", activityCount=" + activityCount +
                ", serviceCount=" + serviceCount +
                ", receiverCount=" + receiverCount +
                ", providerCount=" + providerCount +
                ", permissionCount=" + permissionCount +
                '}';
    }
}
