package vip.devkit.debugkit.kit.crash;

import android.content.Context;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import vip.devkit.debugkit.kit.log.LogCat;

/**
 * Package    Name: vip.devkit.debugkit.kit.crash
 * File       Name: CacheUtils.java
 * Create     Date: 2019/7/9 9:30
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class CacheUtils {

    private CacheUtils() {
    }

    public static boolean saveObject(Context context, String key, Serializable ser) {
        File file = new File(context.getCacheDir() + "/" + key);
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                LogCat.e(e.toString());
            }
        }
        return saveObject(ser, file);
    }

    public static Serializable readObject(Context context, String key) {
        File file = new File(context.getCacheDir() + "/" + key);
        return readObject(file);
    }

    public static boolean saveObject(Serializable ser, File file) {
        FileOutputStream fos = null;
        ObjectOutputStream oos = null;
        try {
            fos = new FileOutputStream(file);
            oos = new ObjectOutputStream(fos);
            oos.writeObject(ser);
            oos.flush();
            return true;
        } catch (IOException e) {
            LogCat.e(e.toString());
            return false;
        } finally {
            if (oos != null) {
                try {
                    oos.close();
                } catch (IOException e) {
                    LogCat.e(e.toString());
                }
            }
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e) {
                    LogCat.e(e.toString());
                }
            }
        }
    }

    public static Serializable readObject(File file) {
        if (file == null || !file.exists() || file.isDirectory()) {
            return null;
        }
        FileInputStream fis = null;
        ObjectInputStream ois = null;
        try {
            fis = new FileInputStream(file);
            ois = new ObjectInputStream(fis);
            return (Serializable) ois.readObject();
        } catch (IOException e) {
            if (e instanceof InvalidClassException) {
                file.delete();
            }
            LogCat.e(e.toString());
            return null;
        } catch (ClassNotFoundException e) {
            LogCat.e(e.toString());
            return null;
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    LogCat.e(e.toString());
                }
            }
            if (ois != null) {
                try {
                    ois.close();
                } catch (IOException e) {
                    LogCat.e(e.toString());
                }
            }
        }
    }
}
