package vip.devkit.debugkit.ui.FileExplorer;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import vip.devkit.debugkit.R;
import vip.devkit.debugkit.adapter.FileFolderAdapter;
import vip.devkit.debugkit.adapter.FileInfoAdapter;
import vip.devkit.debugkit.bean.FileInfoBean;
import vip.devkit.debugkit.kit.log.LogCat;
import vip.devkit.debugkit.ui.base.BaseActivity;
import vip.devkit.debugkit.widget.recyclerview.BaseRecyclerAdapter;

/**
 * Package    Name: vip.devkit.debugkit.ui.FileExplorer
 * File       Name: FileExplorerActivity.java
 * Create     Date: 2019/7/9 14:17
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class FileExplorerActivity extends BaseActivity implements View.OnClickListener, FileInfoAdapter.OnItemClickListener {
    private LinearLayout mLlTitleBack;
    private TextView mTvTitle;
    private TextView mTvTitleRight;
    private LinearLayout mLlTitleRight;
    private LinearLayout mLlCommonLayout;
    private RecyclerView mRlList;
    private RecyclerView mRlFileFolder;
    private LinearLayout mLlFolder;

    private File mCurDir;
    private FileInfoAdapter mAdapter;
    private List<FileInfoBean> mBeanList = new ArrayList<>();
    private FileFolderAdapter mFolderAdapter;
    private List<FileInfoBean> mFolderList = new ArrayList<>();

    @Override
    protected void initData() {

    }

    @Override
    protected int setLayoutId() {
        return R.layout.dk_activity_file_explorer;
    }

    @Override
    protected void initView() {

        mLlTitleBack = (LinearLayout) findViewById(R.id.ll_title_back);
        mLlTitleBack.setOnClickListener(this);
        mTvTitle = (TextView) findViewById(R.id.tv_title);
        mTvTitleRight = (TextView) findViewById(R.id.tv_title_right);
        mLlTitleRight = (LinearLayout) findViewById(R.id.ll_title_right);
        mLlCommonLayout = (LinearLayout) findViewById(R.id.ll_common_layout);
        mRlList = (RecyclerView) findViewById(R.id.rl_list);
        mRlFileFolder = (RecyclerView) findViewById(R.id.rl_file_folder);
        mLlFolder = (LinearLayout) findViewById(R.id.ll_folder);
        mTvTitle.setOnClickListener(this);
        mTvTitleRight.setOnClickListener(this);
        mLlFolder.setOnClickListener(this);

        mFolderAdapter = new FileFolderAdapter(R.layout.dk_item_file_folder, mFolderList);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this);
        layoutManager.setOrientation(RecyclerView.HORIZONTAL);
        mRlFileFolder.setLayoutManager(layoutManager);
        mRlFileFolder.setAdapter(mFolderAdapter);


        mBeanList.addAll(initRootFileInfos(this));
        mAdapter = new FileInfoAdapter(R.layout.dk_item_file, mBeanList);
        mAdapter.setOnItemClickListener(this);
        mRlList.setLayoutManager(new LinearLayoutManager(this));
        mRlList.setAdapter(mAdapter);


    }

    @Override
    protected void setListener() {

        mFolderAdapter.setOnItemClickListener(new BaseRecyclerAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(BaseRecyclerAdapter adapter, View view, int position) {
                FileInfoBean fileInfoBean = mFolderAdapter.getItem(position);
                if (fileInfoBean.file == mCurDir) {
                    LogCat.i("当前文件夹");
                    return;
                }
                setAdapterData(getFileInfos(fileInfoBean.file));
                for (int i = 0; i < mFolderList.size(); i++) {
                    LogCat.i("当前文件夹" + i);
                    if (i > position) {
                        mFolderList.remove(i);
                    }
                }
                mFolderAdapter.notifyDataSetChanged();
            }
        });
    }


    @Override
    public void onItemClick(BaseRecyclerAdapter adapter, View view, int position) {
        FileInfoBean fileInfoBean = mAdapter.getItem(position);
        if (fileInfoBean.file.isFile()) {
            openFile(this,fileInfoBean.file);
        } else {
            mCurDir = fileInfoBean.file;
            setAdapterData(getFileInfos(fileInfoBean.file));
            mFolderList.add(fileInfoBean);
            mFolderAdapter.notifyDataSetChanged();
        }
    }

    private void setAdapterData(List<FileInfoBean> fileInfos) {
        mBeanList.clear();
        if (!fileInfos.isEmpty()) {
            mBeanList.addAll(fileInfos);
        }
        mAdapter.notifyDataSetChanged();

    }

    @Override
    public void onClick(View v) {
        if (v.getId() == R.id.ll_title_back) {
            finish();
        } else if (v.getId() == R.id.ll_folder) {
            mFolderList.clear();
            mFolderAdapter.notifyDataSetChanged();
            mBeanList.clear();
            mBeanList.addAll(initRootFileInfos(this));
            mAdapter.notifyDataSetChanged();
        }
    }


    /**
     * 调用系统应用打开图片
     *
     * @param context context
     * @param file    file
     */
    public static void openFile(Context context, File file) {
        Intent intent = new Intent();
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        intent.setAction(Intent.ACTION_VIEW);
        Uri uri = null;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            // 方式一
            uri = getImageContentUri(context, file);
            // 方式二
            // uri = FileProvider.getUriForFile(context, ".fileProvider", file);
            // intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        } else {
            uri = Uri.fromFile(file);
        }

        String type = null;
        intent.setDataAndType(uri, type);
        try {
            context.startActivity(intent);
        } catch (Exception e) {
            LogCat.e("FileUtil"+ e.getMessage());
            Toast.makeText(context, "找不到打开此文件的应用！", Toast.LENGTH_SHORT).show();
        }
    }
    /**
     * 转换 content:// uri
     *
     * @param imageFile imageFile
     * @return
     */
    public static Uri getImageContentUri(Context context, File imageFile) {
        String filePath = imageFile.getAbsolutePath();
        Cursor cursor = context.getContentResolver().query(
                MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                new String[] { MediaStore.Images.Media._ID },
                MediaStore.Images.Media.DATA + "=? ",
                new String[] { filePath }, null);

        if (cursor != null && cursor.moveToFirst()) {
            int id = cursor.getInt(cursor
                    .getColumnIndex(MediaStore.MediaColumns._ID));
            Uri baseUri = Uri.parse("content://media/external/images/media");
            return Uri.withAppendedPath(baseUri, "" + id);
        } else {
            if (imageFile.exists()) {
                ContentValues values = new ContentValues();
                values.put(MediaStore.Images.Media.DATA, filePath);
                return context.getContentResolver().insert(
                        MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);
            } else {
                return null;
            }
        }
    }

    private List<FileInfoBean> getFileInfos(File dir) {
        List<FileInfoBean> fileInfos = new ArrayList<>();
        if (dir.listFiles() == null) {
            return fileInfos;
        }
        for (File file : dir.listFiles()) {
            FileInfoBean fileInfo = new FileInfoBean(file);
            fileInfos.add(fileInfo);
        }
        return fileInfos;
    }

    private List<FileInfoBean> initRootFileInfos(Context context) {
        List<FileInfoBean> fileInfos = new ArrayList<>();
        fileInfos.add(new FileInfoBean(context.getFilesDir().getParentFile()));
        fileInfos.add(new FileInfoBean(context.getExternalCacheDir()));
        fileInfos.add(new FileInfoBean(context.getExternalFilesDir(null)));
        return fileInfos;
    }

    private boolean isRootFile(Context context, File file) {
        if (file == null) {
            return false;
        }
        return file.equals(context.getExternalCacheDir())
                || file.equals(context.getExternalFilesDir(null))
                || file.equals(context.getFilesDir().getParentFile());
    }
}

