package vip.devkit.debugkit.ui.base;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.FrameLayout;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;


/**
 * Package    Name: vip.devkit.debugkit.ui.base
 * File       Name: BaseFragment.java
 * Create     Date: 2019/7/1 10:44
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public abstract class BaseFragment extends Fragment {

    protected AppCompatActivity mActivity;
    protected View  mRootView;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof BaseActivity) {
            mActivity = (BaseActivity) context;
        } else {
            mActivity = (AppCompatActivity) context;
        }
    }


    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        mRootView = inflater.inflate(setLayoutId(), container, false);
        return mRootView;
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
        initData();
        setListener();
    }



    protected abstract void initData();

    protected abstract int setLayoutId();

    protected abstract void initView(View view);

    protected abstract void setListener();





    protected <T extends View> T getView(int id) {
        return (T) mRootView.findViewById(id);
    }

    /**
     * startActivity
     *
     * @param clazz
     */
    protected void startActivity(Class<?> clazz) {
        Intent intent = new Intent(mActivity, clazz);
        startActivity(intent);
    }


    /**
     * startActivity  putExtras（Bundle）
     *
     * @param clazz
     * @param extras
     */
    protected void startActivity(Class<?> clazz, Bundle extras) {
        Intent intent = new Intent(mActivity, clazz);
        intent.putExtras(extras);
        startActivity(intent);
    }

    /**
     * startActivity  putExtras（Bundle）
     *
     * @param clazz
     * @param requestCode
     * @param extras
     */
    protected void startActivity(Class<?> clazz, int requestCode, Bundle extras) {
        Intent intent = new Intent(mActivity, clazz);
        intent.putExtras(extras);
        startActivityForResult(intent, requestCode);
    }

    /**
     * startActivity  putExtras（Bundle）
     *
     * @param clazz
     * @param code
     */
    protected void startActivity(Class<?> clazz, int code) {
        Intent intent = new Intent(mActivity, clazz);
        startActivityForResult(intent, code);
    }


    /**
     * 重启当前Activity
     */
    private void reStartActivity() {
        Intent intent = mActivity.getIntent();
        mActivity.finish();
        startActivity(intent);
    }





}