package vip.devkit.debugkit.ui.base;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Package    Name: vip.devkit.debugkit.ui.base
 * File       Name: GCBaseExpandableListAdapter
 * Create     Date: 2019/7/1 10:48
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public abstract class GCBaseExpandableListAdapter<Group, Child> extends BaseExpandableListAdapter {

    protected Map<Group, List<Child>> mDatas = new HashMap<>();
    protected List<Group> mGroup = new ArrayList<>();
    protected LayoutInflater mInflater;
    protected Context mContext;
    protected int groupLayoutId;
    protected int childLayoutId;

    public GCBaseExpandableListAdapter(Context context, int groupLayoutId, int childLayoutId, List<Group> mGroup, Map<Group, List<Child>> datas) {
        this.mContext = context;
        this.mGroup = mGroup;
        this.mDatas = datas;
        this.groupLayoutId = groupLayoutId;
        this.childLayoutId = childLayoutId;
        mInflater = LayoutInflater.from(context);
    }


    @Override
    public int getGroupCount() {
        return mGroup.size();
    }

    @Override
    public Group getGroup(int i) {
        return mGroup.get(i);
    }

    @Override
    public int getChildrenCount(int i) {
        return isEmpty(mDatas.get(getGroup(i))) ? 0 : mDatas.get(getGroup(i)).size();
    }

    @Override
    public Child getChild(int i, int i1) {
        List<Child> mGroup = mDatas.get(getGroup(i));
        return mGroup.get(i1);
    }

    @Override
    public long getGroupId(int i) {
        return i;
    }

    @Override
    public long getChildId(int i, int i1) {
        return i1;
    }


    @Override
    public View getGroupView(int position, boolean b, View view, ViewGroup viewGroup) {
        ViewHolder mGroupHolder = ViewHolder.get(mContext, view, viewGroup, groupLayoutId, position);
        convertGroup(mGroupHolder, getGroup(position), position);
        return mGroupHolder.getConvertView();
    }


    @Override
    public View getChildView(int groupPosition, int childPosition, boolean b, View view, ViewGroup viewGroup) {
        ViewHolder childHolder = ViewHolder.get(mContext, view, viewGroup, childLayoutId, childPosition);
        convertChild(childHolder, getGroup(groupPosition), getChild(groupPosition, childPosition), groupPosition, childPosition);
        return childHolder.getConvertView();
    }

    public abstract void convertGroup(ViewHolder holder, Group t, int groupPosition);

    public abstract void convertChild(ViewHolder holder, Group group, Child child, int groupPosition, int childPosition);

    @Override
    public boolean hasStableIds() {
        return true;
    }

    /**
     * 子类点击事件
     *
     * @param i  groupPosition
     * @param i1 childPosition
     * @return
     */
    @Override
    public boolean isChildSelectable(int i, int i1) {
        return true;
    }

    public static <V> boolean isEmpty(List<V> sourceList) {
        return (sourceList == null || sourceList.size() == 0);
    }

    @Override
    public void onGroupExpanded(int groupPosition) {
        super.onGroupExpanded(groupPosition);
    }
}
