package vip.devkit.debugkit.ui.base_url;

import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import vip.devkit.debugkit.R;
import vip.devkit.debugkit.adapter.BaseUrlManagerAdapter;
import vip.devkit.debugkit.kit.log.LogCat;
import vip.devkit.debugkit.ui.base.BaseActivity;

/**
 * Package    Name: vip.devkit.debugkit.ui.base_url
 * File       Name: BaseUrlManagerActivity.java
 * Create     Date: 2019/7/3 9:47
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class BaseUrlManagerActivity extends BaseActivity implements View.OnClickListener,BaseUrlManagerAdapter.OnUrlCheckedListener {
    private LinearLayout mLlTitleBack;
    private TextView mTvTitle;
    private TextView mTvTitleRight;
    private LinearLayout mLlTitleRight;
    private LinearLayout mLlCommonLayout;
    private RecyclerView mRlList;
    private EditText mEtContent;
    private Button mBtnSubmit;

    private BaseUrlManager mBaseUrlManager;
    private UrlInfo mCheckUrlInfo;
    private List<UrlInfo> mUrlList = new ArrayList<>();
    private BaseUrlManagerAdapter mAdapter;

    @Override
    protected void initData() {

    }

    @Override
    protected int setLayoutId() {
        return R.layout.dk_activity_base_url;
    }

    @Override
    protected void initView() {
        mBaseUrlManager = new BaseUrlManager(this);
        mLlTitleBack = (LinearLayout) findViewById(R.id.ll_title_back);
        mTvTitle = (TextView) findViewById(R.id.tv_title);
        mTvTitleRight = (TextView) findViewById(R.id.tv_title_right);
        mLlTitleRight = (LinearLayout) findViewById(R.id.ll_title_right);
        mLlCommonLayout = (LinearLayout) findViewById(R.id.ll_common_layout);
        mRlList = (RecyclerView) findViewById(R.id.rl_list);
        mEtContent = (EditText) findViewById(R.id.et_content);
        mBtnSubmit = (Button) findViewById(R.id.btn_submit);
        mLlTitleBack.setOnClickListener(this);
        mBtnSubmit.setOnClickListener(this);
        mLlTitleRight.setOnClickListener(this);

        mTvTitle.setText("Base URL 管理");

        mTvTitleRight.setText("保存");

        mUrlList.clear();
        mUrlList.addAll(mBaseUrlManager.getUrlInfos());


        mAdapter = new BaseUrlManagerAdapter(R.layout.dk_item_base_url, mUrlList);
        mAdapter.setOnUrlCheckedListener(this);
        mRlList.setLayoutManager(new LinearLayoutManager(this));
        mRlList.setAdapter(mAdapter);


    }

    @Override
    protected void setListener() {

    }


    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.ll_title_back) {
            finish();
        } else if (id == R.id.btn_submit) {
            String addUrl = mEtContent.getText().toString();
            if (TextUtils.isEmpty(addUrl)) {
                Toast.makeText(this, "请输入网络地址", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!addUrl.startsWith("http")) {
                Toast.makeText(this, "请输入正确的网络地址", Toast.LENGTH_SHORT).show();
                return;
            }
            mBaseUrlManager.addUrlInfo(new UrlInfo(addUrl));
            mEtContent.setText("");
            mEtContent.clearFocus();
            mUrlList.clear();
            mUrlList.addAll(mBaseUrlManager.getUrlInfos());
            LogCat.i("mBaseUrlManager:"+mBaseUrlManager.getUrlInfos().size()+"/"+mUrlList);
            mAdapter.notifyDataSetChanged();
        }else if (id==R.id.ll_title_right){
            if (mCheckUrlInfo!=null){
                LogCat.i("mBaseUrlManager setUrlInfo :"+mCheckUrlInfo.getBaseUrl());
                mBaseUrlManager.setUrlInfo(mCheckUrlInfo);
            }
            finish();
        }
    }

    @Override
    public void onChecked(UrlInfo urlInfo) {
        mCheckUrlInfo= urlInfo;
        mAdapter.notifyDataSetChanged();
    }
}
