package vip.devkit.debugkit.ui.debug;

import android.content.Context;
import android.content.pm.IPackageStatsObserver;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageStats;
import android.content.pm.Signature;
import android.os.Build;
import android.os.RemoteException;
import android.util.Log;

import androidx.annotation.Keep;

import java.lang.reflect.Method;
import java.util.Map;
import java.util.TreeMap;

import vip.devkit.debugkit.BuildConfig;
import vip.devkit.debugkit.util.AppUtil;
import vip.devkit.debugkit.util.DeviceUtils;

/**
 * Package    Name: vip.devkit.debugkit.ui.debug
 * File       Name: AppInfoM.java
 * Create     Date: 2019/7/2 15:14
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class AppInfoM {

    @Keep // App 地址
    private String sourceDir;
    @Keep // APK 大小
    private long apkSize;
    public Map getAppInfo(Context context) {
        Map map = new TreeMap();
        PackageInfo pi = DeviceUtils.getPackageInfo(context);
        map.put("包名", "" + pi.packageName);
        map.put("包类型", "" + BuildConfig.BUILD_TYPE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            map.put("版本号", "" + pi.getLongVersionCode());
        } else {
            map.put("版本号", "" + pi.versionCode);
        }
        map.put("sharedUserId", "" + pi.sharedUserId);
        map.put("版本名uid", "" + pi.applicationInfo.uid);
        PackageInfo pia = DeviceUtils.getPackageInfo(context, PackageManager.GET_ACTIVITIES);
        if (pia.activities != null) {
            map.put("activities", "" + pia.activities.length);
            for (int i = 0; i < pia.activities.length; i++) {
                map.put("activities" + i, "" + pia.activities[i].name);
            }
        }
        PackageInfo pip = DeviceUtils.getPackageInfo(context, PackageManager.GET_PERMISSIONS);
        if (pip.permissions != null) {
            map.put("permissions", "" + pip.permissions.length);
        }

        PackageInfo pirp = DeviceUtils.getPackageInfo(context, PackageManager.GET_PERMISSIONS);
        if (pirp.requestedPermissions != null) {
            map.put("requestedPermissions", "" + pirp.requestedPermissions.length);
        }
        if (Build.VERSION.SDK_INT >= 28) {
            PackageInfo pis = DeviceUtils.getPackageInfo(context, PackageManager.GET_GIDS);
            if (pis.signingInfo != null) {
            }
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            map.put("baseRevisionCode", "" + pi.baseRevisionCode);
            map.put("installLocation", "" + pi.installLocation);
        }
        map.put("SDK 目标版本号", "" + String.valueOf(context.getApplicationInfo().targetSdkVersion));
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            map.put("最小支持SDK版本", "" + String.valueOf(context.getApplicationInfo().minSdkVersion));
        }
        map.put("安装包大小", "");
        map.put("安装包 SIGN MD5", "" + AppUtil.getSign(context, pi.packageName));
        map.put("安装包文件MD5", "");
        map.put("首次安装时间", "" + pi.firstInstallTime);
        map.put("更新安装时间", "" + pi.lastUpdateTime);
        map.put("缓存大小", "");

        return map;
    }

    public void queryPacakgeSize(Context context, String pkgName) {
        if (pkgName != null) {
            //使用放射机制得到PackageManager类的隐藏函数getPackageSizeInfo
            PackageManager pm = context.getPackageManager();  //得到pm对象
            try {
                //通过反射机制获得该隐藏函数
                Method getPackageSizeInfo = pm.getClass().getDeclaredMethod("getPackageSizeInfo", String.class, IPackageStatsObserver.class);
                //调用该函数，并且给其分配参数 ，待调用流程完成后会回调PkgSizeObserver类的函数
                getPackageSizeInfo.invoke(pm, pkgName, new PkgSizeObserver());
            } catch (Exception ex) {
                Log.i("build", "NoSuchMethodException"+ex);
                ex.printStackTrace();
            }
        }
    }


    //aidl文件形成的Bindler机制服务类
    public class PkgSizeObserver extends IPackageStatsObserver.Stub {
        /*** 回调函数，
         * @param pStats ,返回数据封装在PackageStats对象中
         * @param succeeded  代表回调成功
         */
        PackageStats mPackageStats;

        @Override
        public void onGetStatsCompleted(PackageStats pStats, boolean succeeded)
                throws RemoteException {
            // TODO Auto-generated method stub
            Log.i("build", "PackageStats"+pStats.codeSize);
            Log.i("build", "PackageStats"+pStats.dataSize);
            Log.i("build", "PackageStats"+pStats.cacheSize);
            this.mPackageStats = pStats;
        }
    }

}