package vip.devkit.debugkit.ui.permission;

import android.app.AppOpsManager;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PermissionInfo;
import android.os.Build;
import android.text.TextUtils;

import androidx.core.app.ActivityCompat;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import vip.devkit.debugkit.util.DeviceUtils;

/**
 * Package    Name: vip.devkit.debugkit.ui.permission
 * File       Name: PermissionData.java
 * Create     Date: 2019/7/3 13:52
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class PermissionData {


    /**
     * @param context
     * @return 自定义权限组
     */
    public List<PermissionInfo> getPermissions(Context context) {
        List<PermissionInfo> list = new ArrayList();
        PackageInfo pip = DeviceUtils.getPackageInfo(context, PackageManager.GET_PERMISSIONS);
        if (pip.permissions != null) {
            list.addAll(Arrays.asList(pip.permissions));
        }
        return list;
    }


    public List<String> getRequestedPermissions(Context context) {
        List<String> list = new ArrayList();
        PackageInfo pip = DeviceUtils.getPackageInfo(context, PackageManager.GET_PERMISSIONS);
        if (pip.requestedPermissions != null) {
            list.addAll(Arrays.asList(pip.requestedPermissions));
        }
        return list;
    }


    public static String getPermissionName(String permission) {
        Map<String, String> map = new TreeMap();

        //威胁需要申请的权限
        map.put("android.permission.BIND_ACCESSIBILITY_SERVICE", "辅助");
        map.put("android.permission.PACKAGE_USAGE_STATS", "应用使用情况");
        map.put("android.permission.BIND_NOTIFICATION_LISTENER_SERVICE", "监听通知栏");
        map.put("android.permission.WRITE_SETTINGS", "修改系统设置");
        map.put("android.permission.SYSTEM_ALERT_WINDOW", "显示在其他应用上面");
        map.put("android.permission.BIND_DEVICE_ADMIN", "设备管理器");
        map.put("android.permission.WRITE_EXTERNAL_STORAGE", "读取/修改/删除SD卡内容");
        map.put("android.permission.READ_EXTERNAL_STORAGE", "读取SD卡中的内容");
        map.put("android.permission.READ_CELL_BROADCASTS", "读取小区广播消息");
        map.put("android.permission.RECEIVE_MMS", "接收文字讯息 (彩信)");
        map.put("android.permission.RECEIVE_WAP_PUSH", "接收文字讯息 (WAP)");
        map.put("android.permission.READ_SMS", "读取您的文字讯息 (短信或彩信)");
        map.put("android.permission.RECEIVE_SMS", "接收文字讯息 (短信)");
        map.put("android.permission.SEND_SMS", "发送短信");
        map.put("android.permission.BODY_SENSORS", "人体传感器 (如心跳速率检测器)");
        map.put("android.permission.ANSWER_PHONE_CALLS", "接听来电");
        map.put("android.permission.READ_PHONE_NUMBERS", "读取电话号码");
        map.put("android.permission.PROCESS_OUTGOING_CALLS", "重新设置呼出的路径");
        map.put("android.permission.USE_SIP", "拨打/接听互联网通话");
        map.put("com.android.voicemail.permission.ADD_VOICEMAIL", "添加语音邮件");
        map.put("android.permission.CALL_PHONE", "直接拨打电话号码");
        map.put("android.permission.WRITE_CALL_LOG", "写入通话记录");
        map.put("android.permission.READ_CALL_LOG", "读取通话记录");
        map.put("android.permission.READ_PHONE_STATE", "设备信息：读取手机状态和身份");
        map.put("android.permission.RECORD_AUDIO", "录音");
        map.put("android.permission.ACCESS_COARSE_LOCATION", "（基于网络的) 大概位置");
        map.put("android.permission.ACCESS_FINE_LOCATION", "精准的 （GPS） 位置");
        map.put("android.permission.GET_ACCOUNTS", "查找设备上的帐户");
        map.put("android.permission.WRITE_CONTACTS", "修改通讯录");
        map.put("android.permission.READ_CONTACTS", "读取您的通讯录");
        map.put("android.permission.CAMERA", "拍摄照片和视频");
        map.put("android.permission.WRITE_CALENDAR", "写日历");
        map.put("android.permission.READ_CALENDAR", "读日历");


        map.put("android.permission.REQUEST_INSTALL_PACKAGES", "8.0+安装权限");
        map.put("android.permission.SYSTEM_ALERT_WINDOW", "6.0+安装权限");

        map.put("android.permission.ANSWER_PHONE_CALLS", "允许您的应用通过编程方式接听呼入电话");
        map.put("android.permission.READ_PHONE_NUMBERS", "权限允许您的应用读取设备中存储的电话号码");

        map.put("android.permission.SYSTEM_ALERT_WINDOW", "安装权限");
        map.put("ACCESS_CHECKIN_PROPERTIES", "");
        return map.get(permission);
    }


    /**
     * 检查某些权限是否全部授予了
     *
     * @param context     上下文对象
     * @param permissions 需要请求的权限组
     */
    public static boolean isHasPermission(Context context, String... permissions) {
        ArrayList<String> failPermissions = getFailPermissions(context, Arrays.asList(permissions));
        return failPermissions == null || failPermissions.isEmpty();
    }

    /**
     * 获取没有授予的权限
     *
     * @param context     上下文对象
     * @param permissions 需要请求的权限组
     */
    static ArrayList<String> getFailPermissions(Context context, List<String> permissions) {
        ArrayList<String> failPermissions = null;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            for (String permission : permissions) {
                // 把没有授予过的权限加入到集合中
                if (context.checkSelfPermission(permission) == PackageManager.PERMISSION_DENIED) {
                    if (failPermissions == null) failPermissions = new ArrayList<>();
                    failPermissions.add(permission);
                }
            }
        } else {
            failPermissions = new ArrayList<>();
        }
        return failPermissions;
    }

    public static boolean hasSelfPermission(Context context, List<String> permissions) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) return true;
        ArrayList<String> failPermissions = null;
        for (String permission : permissions) {
            if (!hasSelfPermission(context, permission)) {
                if (failPermissions == null) failPermissions = new ArrayList<>();
                failPermissions.add(permission);
            }
        }
        return failPermissions == null || failPermissions.isEmpty();
    }

    public static boolean hasSelfPermission(Context context, String permission) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) return true;
        AppOpsManager appOpsManager = (AppOpsManager) context.getSystemService(Context.APP_OPS_SERVICE);
        String op = AppOpsManager.permissionToOp(permission);
        if (!TextUtils.isEmpty(op)) {
            int checkOp = appOpsManager.checkOp(op, android.os.Process.myUid(), context.getPackageName());
            return checkOp == AppOpsManager.MODE_ALLOWED && ActivityCompat.checkSelfPermission(context, permission) == PackageManager.PERMISSION_GRANTED;
        }
        return true;
    }

    public static void ss(){


    }
}