package vip.devkit.debugkit.ui.sysinfo;

import android.content.Context;
import android.os.Build;
import android.util.DisplayMetrics;
import android.util.Log;

import androidx.annotation.RequiresApi;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import vip.devkit.debugkit.util.Comm;
import vip.devkit.debugkit.util.NetUtils;

/**
 * Package    Name: vip.ievkit.iebugkit.ui.sysinfo
 * File       Name: SysInfoKit.java
 * Create     Date: 2019/7/2 9:25
 * Author         : K
 * E-mail         : vip@devkit.vip
 * VersionCode    : V 1.0
 * Describe       ：
 * Code Update    :（author - time）
 * Update Describe：
 */
public class SysInfoKit {

    public SysInfoKit() {

    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public Map<String, String> getInfo() {
        Map<String, String> map = new HashMap();

        String[] s=Build.SUPPORTED_ABIS;

        String apis="";
        for (String ss:s) {
            apis=""+ss+"/";
        }
        apis=apis+Build.CPU_ABI+"/"+Build.CPU_ABI2;
        map.put("品牌", Build.BRAND);
        map.put("厂商", Build.MANUFACTURER);
        map.put("型号", Build.DEVICE);
        map.put("SDK版本", "" + Build.VERSION.SDK_INT);
        map.put("CPU ABI", apis);
        map.put("内核", Build.MANUFACTURER);
        map.put("运行内存", Build.MANUFACTURER);
        map.put("手机存储", Build.MANUFACTURER);
        map.put("处理器", Build.HARDWARE);
        map.put("无线电固件版本", Build.getRadioVersion());
        map.put("硬件识别码", "" + Build.FINGERPRINT);
        map.put("硬件名称", "" + Build.HARDWARE);
        map.put("HOST", "" + Build.HOST);
        map.put("版本", "" + Build.MODEL);
        map.put("硬件序列号", "" + Build.SERIAL);
        map.put("手机制作商", "" + Build.MANUFACTURER);
        map.put("描述Build的标签", "" + Build.TAGS);
        map.put("builder类型", "" + Build.TYPE);
        map.put("USERs", "" + Build.USER);
        map.put("USER", "" + Build.USER + "/" + Build.VERSION.RELEASE + "/");
        map.put("USERs", "" + getNumberOfCPUCores()+"/"+Runtime.getRuntime().availableProcessors());


        return map;
    }


    public Map<String, String> getScreen(Context context) {

        DisplayMetrics dm = context.getResources().getDisplayMetrics();
        int height = dm.heightPixels;
        int width = dm.widthPixels;
        int sw = context.getResources().getConfiguration().smallestScreenWidthDp;
        float density = dm.density;
        int densityDPI = dm.densityDpi;   // 屏幕密度(这里是系统密度）（每寸像素：120/160/240/320,480,640等）
        float xdpi = dm.xdpi;
        float ydpi = dm.ydpi;

        float scaledDensity = dm.scaledDensity;
        Map<String, String> map = new HashMap();

        String dpi = "ldpi";
        if (density == 1.0) {
            dpi = "mdpi";
        } else if (density == 1.5) {
            dpi = "hdpi";
        } else if (density == 2.0) {
            dpi = "xhdpi";
        } else if (density == 3.0) {
            dpi = "xxhdpi";
        }
        // 这样可以计算屏幕的物理尺寸
        float width2 = (width / xdpi) * (width / xdpi);
        float height2 = (height / ydpi) * (width / xdpi);

        float xInch = width / xdpi;
        float yInch = height / ydpi;
        double screenInch = Math.sqrt(Math.pow(xInch, 2) + Math.pow(yInch, 2));
        map.put("密度", "" + densityDPI + "dpi/" + dpi + "/" + density);
        map.put("精确密度", "" + xdpi + "*" + ydpi + "dp");
        map.put("屏幕分辨率", "" + width + "*" + height + "px");
        map.put("可用屏幕分辨率", "" + width + "*" + height + "px");
        map.put("屏幕尺寸", "" + formatDouble(screenInch, 1) + "英寸");
        return map;
    }

    /**
     * Double类型保留指定位数的小数，返回double类型（四舍五入）
     * newScale 为指定的位数
     */
    private static double formatDouble(double d, int newScale) {
        BigDecimal bd = new BigDecimal(d);
        return bd.setScale(newScale, BigDecimal.ROUND_HALF_UP).doubleValue();
    }


    public Map<String, String> getNetInfo(Context context) {
        Map<String, String> map = new TreeMap<>();
        if (!NetUtils.isNetConnected(context)) {
            map.put("网络类型：", "" + NetUtils.getNetworkStateName(context));
        } else {
            map.put("网络类型：", "" + NetUtils.getNetworkStateName(context));
            int state = NetUtils.getNetworkState(context);
            if (state == 0) {
            } else if (state==1) {
                int ipAddress=NetUtils.getWifiInfo(context).getIpAddress();
                String ipString = ((ipAddress & 0xff) + "." + (ipAddress >> 8 & 0xff) + "." + (ipAddress >> 16 & 0xff) + "." + (ipAddress >> 24 & 0xff));
                map.put("WIFI SSID：", "" + NetUtils.getWifiInfo(context).getSSID());
                map.put("WIFI getBSSID：", "" + NetUtils.getWifiInfo(context).getBSSID());
                map.put("WIFI Ip：", "" + ipString);
                map.put("WIFI getLinkSpeed：", "" + NetUtils.getWifiInfo(context).getLinkSpeed());
                map.put("WIFI getNetworkId：", "" + NetUtils.getWifiInfo(context).getNetworkId());
                map.put("WIFI 型号强度：", "" + NetUtils.getWifiInfo(context).getRssi());

            } else if (state == 2 || state == 3 || state == 4 || state == 5) {
                map.put("运行商：", "" + NetUtils.getOperatorName(context));
            }
            map.put("local IP：", "" + NetUtils.getHostIp());
            map.put("IPv4：", "");
            map.put("IPv6：", "");
        }
        map.put("MAC地址：", "" + NetUtils.getMacAddress());
        return map;
    }
    public Map<String, String> getHardware(Context context) {
        Map<String, String> map = new TreeMap<>();


        return map;
    }

    public static int getNumberOfCPUCores() {
        if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.GINGERBREAD_MR1) {
            // Gingerbread doesn't support giving PermissionManagerActivity single application access to both cores, but PermissionManagerActivity
            // handful of devices (Atrix 4G and Droid X2 for example) were released with PermissionManagerActivity dual-core
            // chipset and Gingerbread; that can let an app in the background run without impacting
            // the foreground application. But for our purposes, it makes them single core.
            return 1;  //上面的意思就是2.3以前不支持多核,有些特殊的设备有双核...不考虑,就当单核!!
        }
        int cores;
        try {
            cores = new File("/sys/devices/system/cpu/").listFiles(CPU_FILTER).length;
        } catch (SecurityException e) {
            cores = 0;   //这个常量得自己约定
        } catch (NullPointerException e) {
            cores = 0;
        }
        return cores;
    }

    private static final FileFilter CPU_FILTER = new FileFilter() {
        @Override
        public boolean accept(File pathname) {
            String path = pathname.getName();
            //regex is slow, so checking char by char.
            if (path.startsWith("cpu")) {
                for (int i = 3; i < path.length(); i++) {
                    if (path.charAt(i) < '0' || path.charAt(i) > '9') {
                        return false;
                    }
                }
                return true;
            }
            return false;
        }
    };
}
